from tcsoa.gen.Internal_Workflowaw._2020_12.services import WorkflowService as imp0
from tcsoa.base import TcService


class WorkflowService(TcService):
    getWorkflowTaskAssignments = imp0.getWorkflowTaskAssignments
    updateWorkflowTaskAssignments = imp0.updateWorkflowTaskAssignments
